//
//  ViewController.swift
//  Riego Automatico
//
//  Created by Santiago Pavón Gómez on 15/9/17.
//  Copyright © 2017 UPM. All rights reserved.
//

import UIKit

class ViewController: UIViewController, ParametricFunctionViewDataSource {

    let tankModel = TankModel()
    let trajectoryModel = TrajectoryModel()
    
    @IBOutlet weak var outSpeedTimeFuncView: ParametricFunctionView!
    @IBOutlet weak var outSpeedHeightFuncView: ParametricFunctionView!
    @IBOutlet weak var waterHeightTimeFuncView: ParametricFunctionView!

    @IBOutlet weak var trajectoryFuncView: ParametricFunctionView!
    
    @IBOutlet weak var vtLabel: UILabel!
    @IBOutlet weak var ntLabel: UILabel!
    
    @IBOutlet weak var timeSlider: UISlider!
    @IBOutlet weak var timeLabel: UILabel!
    
    // Instante en el que quiero dibujar la trayectoria del agua desde el
    // deposito hasta la planta a regar.
    var trajectoryTime: Double = 0.0 {
        didSet {
            outSpeedTimeFuncView.setNeedsDisplay()
            outSpeedHeightFuncView.setNeedsDisplay()
            waterHeightTimeFuncView.setNeedsDisplay()
            trajectoryFuncView.setNeedsDisplay()
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()

        outSpeedTimeFuncView.dataSource = self
        outSpeedHeightFuncView.dataSource = self
        waterHeightTimeFuncView.dataSource = self
        trajectoryFuncView.dataSource = self
        
        trajectoryModel.originPos = (0, 2)
        trajectoryModel.targetPos = (1.0, 0)
        
        timeSlider.sendActions(for: .valueChanged)
    }

    // MARK: - Acciones
    
    @IBAction func updateTime(_ sender: UISlider) {
        
        trajectoryTime = Double(sender.value)
        
        let h = tankModel.waterHeightAt(time: trajectoryTime)
        let v = tankModel.waterOutputSpeed(waterHeight: h)
        
        let formatter = NumberFormatter()
        formatter.numberStyle = .decimal
        formatter.maximumFractionDigits = 2
        
        vtLabel.text = ""
        ntLabel.text = ""

        if let fh = formatter.string(from: h as NSNumber) {
            ntLabel.text = "\(fh) m"
        }
        
        if let fv = formatter.string(from: v as NSNumber) {
            vtLabel.text = "\(fv) m/seg"
        }

        timeLabel.text = "Tiempo \(Int(trajectoryTime)) seg"
    }
    
    
    @IBAction func moveUpTank(_ sender: UISwipeGestureRecognizer) {
        
        trajectoryModel.originPos.y += 0.1

        outSpeedTimeFuncView.setNeedsDisplay()
        outSpeedHeightFuncView.setNeedsDisplay()
        waterHeightTimeFuncView.setNeedsDisplay()
        trajectoryFuncView.setNeedsDisplay()

        
    }
    
    @IBAction func moveDownTank(_ sender: UISwipeGestureRecognizer) {
        
        let h = max(0, trajectoryModel.originPos.y - 0.1)
        trajectoryModel.originPos.y = h

        outSpeedTimeFuncView.setNeedsDisplay()
        outSpeedHeightFuncView.setNeedsDisplay()
        waterHeightTimeFuncView.setNeedsDisplay()
        trajectoryFuncView.setNeedsDisplay()
    }
    
    
    @IBAction func moveTank(_ sender: UIPanGestureRecognizer) {
        
        let pos = sender.translation(in: sender.view!)
        
        let h = Double(pos.y) / trajectoryFuncView.scaleY
        
        trajectoryModel.originPos.y -= h

        sender.setTranslation(.zero, in: sender.view)
        
        trajectoryFuncView.setNeedsDisplay()
    }
    @IBAction func updateScaleFuncView(_ sender: UIPinchGestureRecognizer) {
        
        let f  = sender.scale
        
        outSpeedTimeFuncView.scaleX *= Double(f)
        outSpeedTimeFuncView.scaleY *= Double(f)

        outSpeedHeightFuncView.scaleX *= Double(f)
        outSpeedHeightFuncView.scaleY *= Double(f)
        
        waterHeightTimeFuncView.scaleX *= Double(f)
        waterHeightTimeFuncView.scaleY *= Double(f)

        trajectoryFuncView.scaleX *= Double(f)
        trajectoryFuncView.scaleY *= Double(f)

        sender.scale = 1

    }
    
    // MARK: - FunctionViewDataSource
    
    func startIndexFor(_ functionView: ParametricFunctionView) -> Double {
        switch functionView {
        case outSpeedTimeFuncView:
            return 0
        case outSpeedHeightFuncView:
            return 0
        case waterHeightTimeFuncView:
            return 0
        case trajectoryFuncView:
            return 0
        default:
            return 0
        }
    }
    
    
    func endIndexFor(_ functionView: ParametricFunctionView) -> Double {
        switch functionView {
        case outSpeedTimeFuncView:
            return tankModel.timeToEmpty
        case outSpeedHeightFuncView:
            return tankModel.initialWaterHeight
        case waterHeightTimeFuncView:
            return tankModel.timeToEmpty
        case trajectoryFuncView:
            
            let h = tankModel.waterHeightAt(time: trajectoryTime)
            let v = tankModel.waterOutputSpeed(waterHeight: h)
            
            // print("--- Velocidad = \(v)")
            
            trajectoryModel.speed = v
            
            return trajectoryModel.timeToTarget() ?? 1.0
                        
        default:
            return 0
        }
    }
    
    
    func functionView(_ functionView: ParametricFunctionView, pointAt index: Double) -> FunctionPoint {
       
        switch functionView {
        case outSpeedTimeFuncView:
            let time = index
            let h = tankModel.waterHeightAt(time: time)
            let v = tankModel.waterOutputSpeed(waterHeight: h)
            return FunctionPoint(x: time, y: v)
            
        case outSpeedHeightFuncView:
            let h = index
            let v = tankModel.waterOutputSpeed(waterHeight: h)
            return FunctionPoint(x: h, y: v)

        case waterHeightTimeFuncView:
            let time = index
            let h = tankModel.waterHeightAt(time: time)
            return FunctionPoint(x: time, y: h)

        case trajectoryFuncView:

            let h = tankModel.waterHeightAt(time: trajectoryTime)
            let v = tankModel.waterOutputSpeed(waterHeight: h)
            
            // print("--- Velocidad = \(v)")
            
            trajectoryModel.speed = v
            
            let pos = trajectoryModel.positionAt(time: index)
            
            // print("--- Posicion (\(index)) = \(pos.x)  \(pos.y)")
            
            return FunctionPoint(x: pos.x, y: pos.y)

        default:
            return FunctionPoint(x: 0, y: 0)
        }
    }
    
    func pointsOfInterestFor(_ functionView: ParametricFunctionView) -> [FunctionPoint] {
        switch functionView {
        case outSpeedTimeFuncView:
            
            // Velocidad de salida del agua en el instante actual
            let time = trajectoryTime
            let h = tankModel.waterHeightAt(time: time)
            let v = tankModel.waterOutputSpeed(waterHeight: h)
            return [ FunctionPoint(x: time, y: v) ]
            
        case outSpeedHeightFuncView:
            
            // Velocidad de salida del agua en funcion de la altura de agua en el deposito
            let time = trajectoryTime
            let h = tankModel.waterHeightAt(time: time)
            let v = tankModel.waterOutputSpeed(waterHeight: h)
            return [ FunctionPoint(x: h, y: v) ]
            
        case waterHeightTimeFuncView:
            
            // Altura de agua en el deposito en el instante actual
            let time = trajectoryTime
            let h = tankModel.waterHeightAt(time: trajectoryTime)
            
            return [FunctionPoint(x: time, y: h)]
            
        case trajectoryFuncView:
            
            // Posicion del deposito y del tiesto
            return [
                FunctionPoint(x: trajectoryModel.originPos.x, y: trajectoryModel.originPos.y),
                FunctionPoint(x: trajectoryModel.targetPos.x, y: trajectoryModel.targetPos.y)
            ]

        default:
            return []

        }
    }
}

